@@ -15,14 +15,14 @@ import six
 from botocore.exceptions import ClientError
 import json
 
-VERSION = "4.1.1"
+VERSION = "4.1.2"
 
 
-def connect(instance, args):
+def connect(instance: object, args: argparse.Namespace):
     """
-
-    :param instance:
-    :param args:
+    :param instance: <class 'boto3.resources.factory.ec2.Instance'> objects
+        e.g.: ec2.Instance(id='i-0b4e31bc0b')
+    :param args: <class 'argparse.Namespace'>
     :return:
     """
     details = get_details(instance)
@@ -41,13 +41,13 @@ def connect(instance, args):
         pass
 
 
-def _connect(user, instance, args):
+def _connect(user: str, instance: object, args: argparse.Namespace) -> int:
     """
-
-    :param user:
-    :param instance:
-    :param args:
-    :return:
+    :param user:  <class 'str'>
+    :param instance: <class 'boto3.resources.factory.ec2.Instance'> objects
+        e.g.: ec2.Instance(id='i-0b4e31bc0b')
+    :param args: <class 'argparse.Namespace'>
+    :return:  <class 'int'> system(command) std out  int result
     """
     config = {
         'key_path': get_key_path(args, instance),
@@ -68,13 +68,14 @@ def _connect(user, instance, args):
     return system(command)
 
 
-def jump_connect(ssh_instance, jump_server, args):
+def jump_connect(ssh_instance: object, jump_server: object, args: argparse.Namespace) -> bool:
     """
-
-    :param ssh_instance:
-    :param jump_server:
-    :param args:
-    :return:
+    :param ssh_instance: <class 'boto3.resources.factory.ec2.Instance'> objects
+        e.g.: ec2.Instance(id='i-0b4e31bc0b')
+    :param jump_server: <class 'boto3.resources.factory.ec2.Instance'> objects
+        e.g.: ec2.Instance(id='i-0b4e31bc0b')
+    :param args: <class 'argparse.Namespace'>
+    :return:  <class 'bool'>
     """
     details_ssh_instance = get_details(ssh_instance)
     details_jump_server = get_details(jump_server)
@@ -94,13 +95,16 @@ def jump_connect(ssh_instance, jump_server, args):
     return True
 
 
-def _jump_connect(user, ssh_instance, jump_server, args):
+def _jump_connect(user: str, ssh_instance: object, jump_server: object, args: argparse.Namespace) -> int:
     """
-    :param user:
-    :param ssh_instance:
-    :param jump_server:
-    :param args:
-    :return:
+    :param user: <class 'str'>
+    :param ssh_instance: <class 'boto3.resources.factory.ec2.Instance'> objects
+        e.g.: ec2.Instance(id='i-0b4e31bc0b')
+    :param jump_server:<class 'boto3.resources.factory.ec2.Instance'> objects
+        e.g.: ec2.Instance(id='i-0b4e31bc0b')
+    :param args: <class 'argparse.Namespace'>
+        e.g.: ec2.Instance(id='i-0b4e31bc0b')
+    :return:  <class 'int'> system(command) std out  int result
     """
     if '/' in user:
         users = user.split('/')
@@ -135,10 +139,10 @@ def _jump_connect(user, ssh_instance, jump_server, args):
     return system(command)
 
 
-def get_tunnel(args):
+def get_tunnel(args: argparse.Namespace) -> str:
     """
-    :param args:
-    :return:
+    :param args: <class 'argparse.Namespace'>
+    :return: <class 'str'> formatted string for ssh command
     """
     if not args.remote_host:
         return ''
@@ -155,11 +159,11 @@ def get_tunnel(args):
     return "-L '{local_port}:{remote_host}:{remote_port}'".format(**params)
 
 
-def get_details(instance):
+def get_details(instance: object) -> dict:
     """
-
-    :param instance:
-    :return:
+    :param instance: <class 'boto3.resources.factory.ec2.Instance'> objects
+        e.g.: ec2.Instance(id='i-0b4e31bc0b')
+    :return: <class 'dict'> dictionary with selected ec2.Instance properties
     """
     try:
         key_name = instance.key_name.replace(" ", "_")
@@ -168,7 +172,7 @@ def get_details(instance):
         key_name = instance.key_name
         launch_time = instance.launch_time
 
-    return {
+    details = {
         'id': instance.id,
         'name': get_name(instance),
         'type': instance.instance_type,
@@ -182,11 +186,14 @@ def get_details(instance):
         'key_name': key_name
     }
 
+    return details
 
-def get_key_path(args, instance):
+
+def get_key_path(args: argparse.Namespace, instance: object) -> str:
     """
-    :param args:
-    :param instance:
+    :param args: <class 'argparse.Namespace'>
+    :param instance: <class 'boto3.resources.factory.ec2.Instance'> objects
+        e.g.: ec2.Instance(id='i-0b4e31bc0b')
     :return:
     """
     if args.key_path:
@@ -199,31 +206,36 @@ def get_key_path(args, instance):
             return None
 
 
-def get_device_mappings(instance):
+def get_device_mappings(instance: object) -> list:
+    """
+    :param instance: <class 'boto3.resources.factory.ec2.Instance'> objects
+        e.g.: ec2.Instance(id='i-0b4e31bc0b')
+    :return:
+    """
     return flatten([device.values() for device in instance.block_device_mappings])
 
 
-def flatten(array):
+def flatten(array: list):
     list(itertools.chain.from_iterable(array))
 
 
-def get_name(instance):
+def get_name(instance: object) -> str:
     """
-
-    :param instance:
+    :param instance: <class 'boto3.resources.factory.ec2.Instance'> objects
     :return:
     """
-    name = [tag for tag in instance.tags if tag['Key'] == 'Name']
+    name: [dict] = [tag for tag in instance.tags if tag['Key'] == 'Name']
     if not name or 'Value' not in name[0]:
         return 'not-named'
     return name[0].get('Value')
 
 
-def get_instances(args):
+def get_instances(args: argparse.Namespace) -> list:
     """
-
-    :param args:
-    :return:
+    :param args: <class 'argparse.Namespace'>
+    Simple object for storing attributes. Implements equality by attribute names and values,
+    and provides a simple string representation.
+    :return: list of objects <class 'boto3.resources.factory.ec2.Instance'>
     """
     # AWS_ACCESS_KEY_ID - Specifies an AWS access key associated with an IAM user or role.
     # AWS_SECRET_ACCESS_KEY - Specifies the secret key associated with the access key.
@@ -327,9 +339,7 @@ def get_instances(args):
 
 
 def main():
-    """
-    :return:
-    """
+    """void main"""
     parser = create_parser()
     args = parser.parse_args()
 
@@ -343,7 +353,7 @@ def main():
         exit(0)
 
     try:
-        instances = get_instances(args)
+        instances: list = get_instances(args)
     except(exceptions.EndpointConnectionError, ValueError):
         print('"{}" is an invalid Region.'.format(args.region))
         exit(0)
@@ -386,9 +396,10 @@ def main():
         select_instance(args, instances)
 
 
-def display_instances(instances):
+def display_instances(instances: list):
     """
-    :param instances:
+    :param instances: list of objects <class 'boto3.resources.factory.ec2.Instance'>
+        e.g.: [ec2.Instance(id='i-0b4e31bc0b'), ec2.Instance(id='i-0f5e60ce9f')]
     :return:
     """
     table_col_names = ['Name', 'Public IP', 'Private IP', 'Zone', 'Key Name']
@@ -402,10 +413,12 @@ def display_instances(instances):
     print('')
 
 
-def select_instance(args, instances):
+def select_instance(args: argparse.Namespace, instances: list):
     """
-    :param args:
-    :param instances:
+    :param args: <class 'argparse.Namespace'> simple object for storing attributes
+    :param instances: <class 'list'>
+    List of <class 'boto3.resources.factory.ec2.Instance'> objects
+        e.g.: [ec2.Instance(id='i-0b4e31bc0b'), ec2.Instance(id='i-0f5e60ce9f')]
     :return:
     """
 
@@ -417,7 +430,7 @@ def select_instance(args, instances):
             server_selection = [int(x) for x in six.moves.input().split()]
 
         if len(server_selection) == 1:
-            server_selection[0] = server_selection[0] - 1  # server selection option starts with 1, not 0
+            server_selection[0] = server_selection[0] - 1  # server selection numbers start at 1, shift to match index
             if instances[server_selection[0]].public_ip_address is None:
                 for instance in instances:
                     if instance.key_name == instances[server_selection[0]].key_name:
@@ -462,10 +475,30 @@ def select_instance(args, instances):
         exit(0)
 
 
-def create_parser():
-    """
+def get_private_key(args: object, environment: int = 'prod') -> object:
+    region = args.region
+    secret_name = f'/{environment}/ssh'
+    client = boto3.client('secretsmanager', region_name='{}'.format(region))
 
-    :return:
+    try:
+        get_secret_value_response = client.get_secret_value(
+            SecretId=secret_name
+        )
+    except ClientError as e:
+        raise e
+
+    else:
+        secret = get_secret_value_response['SecretString']
+
+        secret_dict = json.loads(secret)
+        private_key = secret_dict['PrivateKey']
+
+        return private_key
+
+
+def create_parser() -> argparse.ArgumentParser:
+    """
+    :return: class 'argparse.ArgumentParser'
     """
     parser = argparse.ArgumentParser(description="""
                                         SSH, 'ProxyJump', create tunnels, and run commands on your AWS Linux/Unix
